@@ -87,7 +87,7 @@
 #' @useDynLib regsem
 #' @import Rcpp
 #' @import lavaan
-#' @importFrom stats cov na.omit nlminb pchisq rnorm runif sd uniroot var
+#' @importFrom stats cov na.omit nlminb pchisq rnorm runif sd uniroot var weighted.mean
 #' @export
 #' @examples
 #' library(lavaan)
@@ -119,6 +119,19 @@ regsem = function(model,lambda=0,alpha=0,type="none",data=NULL,optMethod="nlminb
                  missing="listwise"){
 
 
+  if(missing=="fiml" & is.null(model@SampleStats@missing[[1]])){
+    stop("need to change missing=fiml in lavaan")
+  }
+
+  if(gradFun != "none" & missing=="fiml"){
+    stop("only gradFun = none is supported with missing data")
+  }
+
+  if(model@Data@nobs[[1]] != model@Data@norig[[1]]){
+    warning("regsem is currently not working well in the presence of missing data")
+  }
+
+
     if(gradFun=="norm"){
       stop("Only recommended grad function is ram or none at this time")
     }
@@ -136,22 +149,44 @@ regsem = function(model,lambda=0,alpha=0,type="none",data=NULL,optMethod="nlminb
       stop("regsem currently does not allow equality constraints")
     }
 
+
+  if(model@SampleStats@ngroups > 1){
+    stop("regsem currently does not work with multiple group models")
+  }
+
+
+
+
+
+
     nvar = model@pta$nvar[[1]][1]
     nfac = model@pta$nfac[[1]][1]
 
     if(missing=="listwise"){
       calc_fit = "cov"
       nobs = model@SampleStats@nobs[[1]][1]
-      SampMean <- model@SampleStats@mean[][[1]]
 
-      if(length(model@ParTable$op[model@ParTable$op == "~1"]) > 0){
+
+
+
+
+
+
+      if(extractMatrices(model)$mean == TRUE){
+        mm = extractMatrices(model)$A[,"1"]
+
+        SampMean <- model@SampleStats@mean[][[1]]
+        ss = match(names(mm[mm > 0]),model@Data@ov$name)
+        SampMean[-c(ss)] = 0
+
         SampCov1 <- model@SampleStats@cov[][[1]]
         SampCov2 <- SampCov1 + SampMean%*%t(SampMean)
         # try changing size of SampCov
         SampCov3 = cbind(SampCov2,SampMean)
         SampCov = rbind(SampCov3,append(SampMean,1))
-      }else if(length(model@ParTable$op[model@ParTable$op == "~1"]) == 0){
+      }else if(extractMatrices(model)$mean == FALSE){
         SampCov <- model@SampleStats@cov[][[1]]
+        SampMean = NULL
       }
 
       #for grad ram with mean
@@ -160,18 +195,20 @@ regsem = function(model,lambda=0,alpha=0,type="none",data=NULL,optMethod="nlminb
     }else if(missing=="fiml"){
       #stop("FIML is currently not supported at this time")
       calc_fit = "ind"
-      if(is.null(data)==TRUE){
-        stop("Dataset needs to be provided for missing==fiml")
-      }
+      SampCov <- model@SampleStats@cov[][[1]]
+
+      nobs = model@SampleStats@nobs[[1]][1]
+     # if(is.null(data)==TRUE){
+     #   stop("Dataset needs to be provided for missing==fiml")
+     # }
 
 
-      if(length(model@ParTable$op[model@ParTable$op == "~1"]) == 0){
-        stop("meanstructure needs to be equal to TRUE for FIML")
-      }
+     # if(length(model@ParTable$op[model@ParTable$op == "~1"]) == 0){
+     #   stop("meanstructure needs to be equal to TRUE for FIML")
+     # }
 
     }
     #SampCov <- fitted(model)$cov
-
     #SampMean <- rep(0,nvar)
 
     type2 = 0
@@ -300,7 +337,9 @@ if(fac.type=="cfa"){
   if(calc == "normal"){
     calc = function(start){
          mult = rcpp_RAMmult(par=start,A,S,S_fixed,A_fixed,A_est,S_est,F,I)
-         #mult = RAMmult(par=start,A,S,F,A_fixed,A_est,S_fixed,S_est)
+         #print(mult)
+         #mult2 = RAMmult(par=start,A,S,F,A_fixed,A_est,S_fixed,S_est)
+         #print(mult2)
          pen_vec = c(mult$A_est22[A %in% pars_pen],mult$S_est22[S %in% pars_pen])
          if(type=="diff_lasso"){
            pen_diff = pen_vec - diff_par
@@ -313,8 +352,11 @@ if(fac.type=="cfa"){
            fit
          }else if(calc_fit=="ind"){
            #stop("Not currently supported")
-           fit = fiml_calc(ImpCov=mult$ImpCov,data=data,
-                           Areg=mult$A_est22,lambda,alpha,type,pen_vec,nvar)
+           #print(mult$ImpCov)
+           fit = fiml_calc(ImpCov=mult$ImpCov,mu.hat=model@SampleStats@missing.h1[[1]]$mu,
+                           h1=model@SampleStats@missing.h1[[1]]$h1,
+                           Areg=mult$A_est22,lambda,alpha,type,pen_vec,nvar,
+                           lav.miss=model@SampleStats@missing[[1]])
          }
 
     }
@@ -702,7 +744,7 @@ if(optMethod=="nlminb"){
 
 
 
-    if(length(model@ParTable$op[model@ParTable$op == "~1"]) > 0 & missing=="listwise"){
+    if(extractMatrices(model)$mean==TRUE & missing=="listwise"){
       Imp_Cov = Imp_Cov1[1:(nrow(Imp_Cov1)-1),1:(ncol(Imp_Cov1)-1)] - SampMean %*% t(SampMean)
     }else{
       Imp_Cov = Imp_Cov1
@@ -710,6 +752,8 @@ if(optMethod=="nlminb"){
 
     res$Imp_Cov <- Imp_Cov
 
+    res$logl_sat <- as.numeric(fitmeasures(model)["unrestricted.logl"])
+
     #res$grad <- grad(as.numeric(pars.df))
     #### KKT conditions #####
     if(gradFun=="none"){
@@ -802,11 +846,15 @@ if(optMethod=="nlminb"){
       #res$fit = 0.5*(log(det(Imp_Cov1)) + trace(SampCov %*% solve(Imp_Cov1)) -
        #              log(det(SampCov))  - nvar)
       res$fit = rcpp_fit_fun(Imp_Cov1, SampCov,type2=0,lambda=0,pen_vec=0,pen_diff=0)
+    }else if(missing == "fiml" & type == "none"){
+      res$fit = res$out$objective
     }
 
     SampCov <- model@SampleStats@cov[][[1]]
     res$SampCov = SampCov
 
+    res$data <- as.data.frame(model@Data@X)
+
     res$coefficients <- round(pars.df,3)
     res$nvar = nvar
     res$N = nobs
@@ -822,7 +870,7 @@ if(optMethod=="nlminb"){
 
     #res$grad <- grad(res$par.ret)
 
-    #res$hess <- hess(res$par.ret)
+   # res$hess <- hess(as.numeric(res$par.ret))
 
 
     if(res$convergence != 0){
